#!/usr/bin/env python2

import numpy as np
import sys
from lxml import etree
import rrdtool
import copy
import logging

class RRDDatastore(object):

    def __init__(self, host, service, track):
        """USAGE: rrd_datastore((str)<host>, (str)<service>, (str)<track>)

        Sets this class's self variables to hold the host and service
        information. Also sets the guessed filenames for the RRD and XML.
        """

        self.host = host
        self.service = service
        self.track = track

        rrd, xml = RRDDatastore.guess_filename(host, service)

        self.rrd = rrd
        self.xml = xml

        self.track_number = str(self.get_track_number())

        text = open("/tmp/test.log", "a")
        text.write(self.rrd + "\n")
        text.close()

        self.minimum = 0

        self.dataset = None

    def get_track_number(self):
        """Gets the track number for the given RRD track. """

        root = etree.parse(self.xml)
        for ds in root.iter('DATASOURCE'):
            if ds.find('NAME').text == self.track:
                return ds.find('DS').text

    def parse_rrd_file(self, start=1, end='now'):
        """USAGE: parse_rrd_file([(RRD time)<range> (RRD time)<end>])

        Sets information inside of this class about the RRD of choice.
        """

        #~ Record the range for later use
        self.start = start

        #~ Initialize data by unpacking it from rrdtoolfetch
        rrdinfo, dsinfo, rrddata = rrdtool.fetch(
            self.rrd, 'AVERAGE', '-s', 'e-%dw' % start, '-e', end)

        #~ Change the timeseries organization
        organized_data = zip(*rrddata)

        #~ Add each individual dataset to the self.list_of_datasets, also label each
        for dsname, dataset in zip(dsinfo, organized_data):
            if self.track_number == dsname:
                tmprrddata = rrdinfo + (dsname, )
                tmp = Dataset(*tmprrddata)
                tmp.set_dataset(dataset)
                self.dataset = tmp

    def parse_xml_file(self):
        """USAGE: parse_xml_file()

        Parses XML file for the given RRD and assigns information to the RRDs.
        """

        xml = etree.parse(self.xml)
        datasources = xml.findall('DATASOURCE')
        for ds in datasources:
            if ds.find('NAME').text == self.track:
                self.dataset.label = ds.find('LABEL').text
                self.dataset.unit = ds.find('UNIT').text
                if self.dataset.unit is None:
                    self.dataset.unit = '%'
                self.dataset.warn = ds.find('WARN').text
                self.dataset.crit = ds.find('CRIT').text
                self.dataset.host = self.host
                self.dataset.service = self.service

    def __str__(self):
        return self.dataset.__repr__()

    @staticmethod
    def guess_filename(host, service):
        """USAGE: guess_filename( (str)<host> , (str)<service> )

        Guesses the filenames for the XML and RRD file given the host 
        and serivce name. Uses Nagios LLC defaults for Nagios XI.
        """

        base = "/usr/local/nagios/share/perfdata/%s/%s" % (host, service)

        rrd = base + '.rrd'
        xml = base + '.xml'

        return rrd, xml


class Dataset(object):

    def __init__(self, start, end, step, ds):
        """USAGE: RRDDataset((int)<start>, (int)<end>, (int)<step>)

        Initialize general data about the RRD dataset.
        """

        self.start = start
        self.end = end
        self.step = step
        self.ds = ds

        self.label = ''
        self.warn = ''
        self.crit = ''
        self.unit = ''

        self.period = 0

        self.fill_value = None
        self.dataset = None
        self.steps_in_period = None
        self.minimum = None

    def set_dataset(self, data):
        """USAGE: set_dataset((list)<data>)

        Use this as a mutator for the dataset contained by this class. 
        Makes the necessary masked array of the given dataset.
        
        """
        tmp_array = np.ma.masked_array(data)
        tmp_array = np.ma.masked_equal(tmp_array, None)
        self.fill_value = np.ma.mean(tmp_array.compressed())
        self.dataset = copy.deepcopy(tmp_array)

    def get_y_values(self, safe=True):
        """USAGE: get_y_values( [(Boolean)<safe>] )

        Returns the dataset. It defaults to returning a safe dataset.
        However if you want the unfilled dataset, simply pass 
        safe = False.
        
        """
        if safe:
            fill_value = 0
            if not self.fill_value is None:
                fill_value = self.fill_value

            returned_set = self.dataset.filled(fill_value)
            new_set = np.ma.clip(returned_set, 0, np.inf)

            if np.isnan(np.sum(new_set)):
                for i, x in enumerate(new_set):
                    if np.isnan(x):
                        new_set[i] = 0

            return new_set
        else:
            return self.dataset
    get_dataset = get_y_values

    def get_integrity(self):
        """Gets the fraction of valid non-masked values in the dataset. """
        masked_count = np.ma.count_masked(self.dataset)
        return 1 - (float(masked_count) / len(self.dataset))


    def get_x_values(self):
        """USAGE: get_x_values( [(int)<start> (int)<end>] )

        Returns a dataset's points' x values.
        """
        return np.ma.arange(self.start, self.end, self.step)
    get_independent_variable = get_x_values

    def get_xy_pairs(self, safe=True):
        """USAGE: get_xy_pairs( [(Boolean)<safe>] )

        Returns a dataset's points as a sequence.
        """
        return zip(self.get_x_values(), self.get_y_values(safe))


    def get_highcharts_dataset(self):
        d = self.get_y_values().tolist()

        return {
            "type": "area",
            "name": "%s - %s" % (self.label, self.unit),
            "pointStart": int(self.start) * 1000,
            "pointInterval": int(self.step) * 1000,
            "data": d,
        }


    def set_min(self, min):
        """ Sets a dataset's minimum value. """
        self.minimum = minimum

    def get_mean(self):
        """ Gets a dataset's maximum value. """
        m = np.mean(self.get_dataset())
        if np.isnan(m):
            return 0
        else:
            return m

    def get_max(self):
        """ Gets a dataset's maximum value. """
        m = max(self.get_dataset())
        if np.isnan(m):
            return 0
        else:
            return m

    def set_period(self, period=1):
        """USAGE: set_period( [(int)period] )

        Sets the value of this classes period to number of steps in 
        period.
        
        """
        SECONDS_IN_WEEK = 604800

        self.period = period * SECONDS_IN_WEEK
        self.steps_in_period = int(self.period / self.step)

    def get_value_on_date(self, date):
        """Gets the value on a date, or the first value after the date. """

        x = self.get_independent_variable()
        y = self.get_dataset()

        if date < x[0]:
            return 'Date happens before extrapolation begins.'
        if date > x[-1]:
            return 'Date happens after extrapolation ends.'
        for i, d in enumerate(x):
            if date <= d:
                return y[i]

    def __str__(self):
        return self.service + ' on ' + self.host
